function [cl, cd, cm] = getcoeffs(self, aoa, re, extrapMethod)
    % getcoeffs Interpolate polars and returns coefficients for given AoA and Reynolds.
    %   This function returns does mesh interpolation of the polars and returns the three
    %   coefficients (lift, drag, moment) for the input angles of attacks and reynolds.
    %   It is possible to specify an extrapolation method as well if results outside the original
    %   polars need to be retrieved. In that case, it is strongly suggested to use the 'Viterna'
    %   method as it is the only one that tries to emulate the proper aerodynamic behavior of the
    %   airfoil in the extreme ranges. If no extrapMethod is specified, self.extrapMethod will be
    %   used (default is 'spline').
    %
    % Note
    %   The inputs can be supplied as vectors of same size. In that case, the output will be the
    %   coefficients for each couple of (aoa,reynolds)
    %
    % Note
    %   This function uses 'spline' interpolation. This is because
    %     - It is closer to the true curve anyway
    %     - MATLAB automatically extrapolates to find values out of bounds, including for 2D interp.
    % -----
    % Usage
    %   [cl, cd, cm] = Airfoil.getcoeffs(aoa, re)
    %
    % Input
    %   aoa          : Angles of attacks, [rad]
    %   re           : Reynolds numbers, [-]
    %   extrapMethod : Extrapolation method ('none', 'spline', 'viterna')
    %
    % Output
    %   cl : Lift coefficients, [-]
    %   cd : Drag coefficients, [-]
    %   cm : Moment coefficients, [-]
    %
    % Examples
    %   [cl, cd, cm] = Airfoil.getcoeffs(0.17, 1e6)
    %   [cl, cd, cm] = Airfoil.getcoeffs([0.17,0.18], [1e5,1e6])
    %
    % See also: Airfoil.
    %
    % <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (online)</a>

    % -------------------------------------------
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
    % Issues: https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues
    % ----------------------------------------------------------------------------------------------

    narginchk(3, 4);

    % Defaults and constants
    ALLOWED_EXTRAP = {'none', 'spline', 'viterna'};
    EXTENDED_POLARS = {'viterna'};

    % Input validation
    if nargin == 3
        extrapMethod = self.extrapMethod;
    end

    validateattributes(aoa, {'double'}, {'row', 'nonempty'}, mfilename(), 'aoa');
    validateattributes(re, {'double'}, {'row', 'nonempty', 'size', size(aoa)}, mfilename(), 're');
    extrapMethod = validatestring(extrapMethod, ALLOWED_EXTRAP, mfilename(), 'extrapMethod');

    if any(strcmpi(extrapMethod, EXTENDED_POLARS))
        polarType = 'extended';
        if isempty(self.Ext.aoa)
            self.extendpolar;
        end
    else
        polarType = 'original';
    end

    % Interpolate coefficients
    % -------------------------------------------

    % Select correct polar
    Pol = selectpolar(self, polarType);
    if numel(self.reynolds) == 1
        cl = checkandinterp1(Pol, 'cl', aoa, extrapMethod);
        cd = checkandinterp1(Pol, 'cd', aoa, extrapMethod);
        cm = checkandinterp1(Pol, 'cm', aoa, extrapMethod);
    else
        % Create meshgrid with the polar data points
        [X, Y] = meshgrid(Pol.aoa(:, 1), Pol.reynolds);

        % Create meshgrid with the actual values for each element
        [Xq, Yq] = meshgrid(aoa', re);

        % 2D interpolation
        cl = checkandinterp2(X, Y, Pol.cl, Xq, Yq, extrapMethod);
        cd = checkandinterp2(X, Y, Pol.cd, Xq, Yq, extrapMethod);
        cm = checkandinterp2(X, Y, Pol.cm, Xq, Yq, extrapMethod);
    end

end

function Polar = selectpolar(self, polarType)
    % selectpolar Select the correct polar to use for the interpolation.

    if strcmp(polarType, 'original')
        tmpPol = self;
    elseif strcmp(polarType, 'extended')
        tmpPol = self.Ext;
    end

    Polar = struct('reynolds', self.reynolds, ...
                   'aoa', tmpPol.aoa, ...
                   'cl', tmpPol.cl, ...
                   'cd', tmpPol.cd, ...
                   'cm', tmpPol.cm);

end

function out = checkandinterp1(Pol, field, aoa, extrapMethod)
    % checkandinterp1 Checks if field is not empty and interpolate (1D)
    %   If field is empty, return NaN vector of the size of AoA

    val = Pol.(field);

    if ~isempty(val)
        if strcmpi(extrapMethod, 'none')
            out = interp1(Pol.aoa, val, aoa, 'spline', NaN);
        else
            out = interp1(Pol.aoa, val, aoa, 'spline');
        end
    else
        out = nan(size(aoa));
    end

end

function out = checkandinterp2(X, Y, val, Xq, Yq, extrapMethod)
    % checkandinterp1 Checks if field is not empty and interpolate (1D)
    %   If field is empty, return NaN vector of the size of AoA

    warning('off', 'MATLAB:interp2:NaNstrip'); % Warning if original polars contain NaN already

    if ~isempty(val)
        if strcmpi(extrapMethod, 'none')
            out = diag(interp2(X, Y, val', Xq, Yq, 'spline', NaN));
        else
            out = diag(interp2(X, Y, val', Xq, Yq, 'spline'));
        end
    else
        out = nan(length(Xq));
    end

    warning('on', 'MATLAB:interp2:NaNstrip');
end
