function Obj = polypolar(polyCl, polyCd)
    % POLYPOLAR  Create a polar object using polynomial coefficients for cl and cd
    %   This static method can be used in place of a normal constructor.
    %   This will create a Polar object based on the polynomial coefficients given as input.
    %   The polar created will cover the full range of angles of attack ([-pi, pi] rad), with a step
    %   of 0.5 deg between two points.
    % -----
    %
    % Usage:
    %   Polar.polypolar(polyCl,polyCd) returns a polar object with values equal to the coefficients.
    %
    % Inputs:
    %   polyCl : polynomial coefficients for cl (see polyval function for formatting), [1/deg]
    %   polyCd : polynomial coefficients for cd (see polyval function for formatting), [1/deg]
    %
    % Output:
    %   Obj: Polar object polynomial values for aoa, cl and cd an default values otherwise.
    %
    % Example:
    %   NewPolar = af_tools.Polar.polypolar([0.001, 2*pi, 0], [0.1, 0, 0.002]);
    %
    % See also: af_tools.Polar.
    %
    % <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (online)</a>

    % -------------------------------------------
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
    % Issues: https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues
    % ----------------------------------------------------------------------------------------------
    % Defaults and constants
    DEF.AOA_STEP = 0.5;
    DEF.AIRFOIL = {'Polynomial'};
    DEF.REYNOLDS = 1e6;
    DEF.MACH = 0;

    % ---------------------------------------
    % Construct empty object first
    Obj = af_tools.Polar();

    % Get lift and drag values
    aoa_deg = (-180:DEF.AOA_STEP:180)';
    Obj.aoa = deg2rad(aoa_deg);
    Obj.cl = polyval(polyCl, aoa_deg);
    Obj.cd = polyval(polyCd, aoa_deg);

    % Set other properties to default values
    Obj.airfoil = DEF.AIRFOIL;
    Obj.reynolds = DEF.REYNOLDS;
    Obj.mach = DEF.MACH;

end
