function [alpha_s, c_l_s, c_d_s] = findstall(varargin)
    % FINDSTALL Finds the stall point and returns the corresponding AOA, CL and CD.
    %   This function looks for the highest local maximm in the CL curve to determine the position
    %   of the stall point.
    %
    %   FINDSTALL accepts inputs in the form of a Polar structure (generated with XF2MAT) or values
    %   for angles of attack and their associated cl and cd. While alpha and cl are required for
    %   this function, cd is optional. If it is provided, the function will return the drag value at
    %   stall as well, else the cd_s will be an empty vector.
    %
    %   If the input is specified as aoa, cl and cd, the three arrays must have the same number of
    %   rows (one for each angle of attack). The columns indicate values for different polars
    %   (different Mach, different Reynolds,...).
    %   For example, if the cl values are given for 2 different Reynols and there are 100 values for
    %   the aoa, cl and cd must be [100 x 2].
    %
    %   If extended polars are used as input (see EXTENDPOLAR), the function will only look for the
    %   zero-lift angle in the range of [0, 30] deg.
    % -----
    %
    % Usage:
    %   [ALPHA_S, CL_S, CD_S] = FINDSTALL prompts the user to input a Polar structure and determines
    %   the stall angle of attack (ALPHA_S), the lift coefficient value at stall (CL_S) and the drag
    %   coefficient at stall (CD_S).
    %
    %   [ALPHA_S, CL_S, CD_S] = FINDSTALL(POLAR) uses the POLAR strcture given as input instead of
    %   prompting the user to select the file.
    %
    %   [ALPHA_S, CL_S, CD_S] = FINDSTALL(ALPHA, CL, CD) uses directly the values given in in ALPHA,
    %   CL and CD to determine the stall.
    %
    % Inputs:
    %   Polar  : Polar structure (e.g. output from XF2MAT)
    %   alpha  : Vector or array with angles of attack [aoas x 1] or [aoas x nbPolar]
    %   cl     : Array with lift coefficients [aoas x nbPolar]
    %   cd     : Array with drag coefficients [aoas x nbPolar]
    %
    % Output:
    %   alpha_s : Stall angle of attack, IN RADIANS
    %   cl_s    : Lift at stall angle of attack
    %   cd_s    : Drag at stall angle of attack (empty if no cd as input)
    %
    % Example:
    %   FINDSTALL
    %   FINDSTALL(Polar)
    %   FINDSTALL(alpha, cl)
    %   FINDSTALL(alpha, cl, cd)
    %
    % See also: XF2MAT, FINDZEROLIFT, FINDCLLINEARRANGE.
    %
    % <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (online)</a>

    % -------------------------------------------
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
    % Issues: https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues
    % ----------------------------------------------------------------------------------------------

    narginchk(0, 3);

    % Import other functions from this package
    import af_tools.utils.*

    % Defaults and constants
    ORIGINAL_RANGE = deg2rad([0; 30]);  % Look for values in this range only
    OPTION_LIST = {};   % No options for this function

    % Extract and validate the inputs
    [alpha, c_l, c_d] = parsepolarinputs(OPTION_LIST, varargin{:});

    % Trim extended polars to original range
    % (avoid issues if extended polars are used as input)
    [alpha, c_l, c_d] = trimtorange(ORIGINAL_RANGE, alpha, c_l, c_d);

    % -------------------------------------------
    % Find stall for each polar

    [alpha_s, c_l_s, c_d_s] = stallfinder(alpha, c_l, c_d);

end

% --------------------------------------------------------------------------------------------------
function [alpha_s, c_l_s, c_d_s] = stallfinder(alpha, c_l, c_d)
    % STALLFINDER Finds the stall point and returns the corresponding AOA, CL and CD

    PLOT_POLAR_IF_ERROR = true; % Plot the polar if error so user can see why it fails

    % Initialization
    alpha_s = zeros(1, size(c_l, 2));
    c_l_s = zeros(1, size(c_l, 2));
    if isempty(c_d)
        c_d_s = c_d;
    else
        c_d_s = zeros(1, size(c_l, 2));
    end

    % Loop for every polar data (every column of cl)
    for i = 1:size(c_l, 2)

        % Find all local maxima in c_l, then assume the highest one is the stall point
        locMax = islocalmax(c_l(:, i));
        [~, absMaxIdx] = max(locMax);

        if ~any(locMax)
            if PLOT_POLAR_IF_ERROR
                figure('Name', 'Debug: Polar with no stall');
                plot(rad2deg(alpha(:, i)), c_l(:, i));
                title('Polar with no clear stall peak');
                xlabel('AOA [deg]');
                ylabel('C_l [-]');
                grid on;
            end
            error('MATLAB:findstall:noStallFound', ...
                  ['The stall point could not be determined as no local maximum was detected. '...
                   'Please provide a polar that goes a bit after the stall.\n' ...
                   'See the attached plot for more details']);
        end

        alpha_s(i) = alpha(absMaxIdx, i);
        c_l_s(i) = c_l(absMaxIdx, i);
        if ~isempty(c_d)
            c_d_s(i) = c_d(absMaxIdx, i);
        end

    end

end
