function plotpolars(varargin)
    % PLOTPOLARS Plot various polars.
    %   Plots the typical polars curves of an airfoil.
    %
    %   PLOTPOLARS accepts inputs in the form of a Polar structure (generated with XF2MAT) or values
    %   for angles of attack and their associated cl, cd and cm.
    %
    %   If the input is specified as aoa, cl and cd, the three arrays must have the same number of
    %   rows (one for each angle of attack). The columns indicate values for different polars
    %   (different Mach, different Reynolds,...).
    %   For example, if the cl values are given for 2 different Reynols and there are 100 values for
    %   the aoa, cl and cd must be [100 x 2].
    %
    % Plots:
    %   - Cl - aoa
    %   - Cd - aoa
    %   - Cm - aoa
    %   - Cl/Cd - aoa
    %   - Cl - Cd
    %
    % <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (README)</a>
    % <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues">Report an issue</a>
    % -----
    %
    % Usage:
    %   PLOTPOLARS prompts the user to input a Polar structure and plots the various curves.
    %
    %   PLOTPOLARS(POLAR) uses the POLAR strcture given as input instead of prompting the user to
    %   select the file.
    %
    %   PLOTPOLARS(ALPHA, CL) uses directly the values provided by ALPHA and CL to plot the
    %   available curves.
    %
    %   PLOTPOLARS(ALPHA, CL, CD) uses directly the values provided by ALPHA and CL and CD to plot
    %   the available curves.
    %
    %   PLOTPOLARS(ALPHA, CL, CD, CM) uses directly the values provided by ALPHA, CL, and CM to plot
    %   all possible curves.
    %
    % Inputs:
    %   Polar  : Polar structure (e.g. output from XF2MAT)
    %   alpha  : Vector or array with angles of attack [aoas x 1] or [aoas x nbPolar]
    %   cl     : Array with lift coefficients [aoas x nbPolar]
    %   cd     : Array with drag coefficients [aoas x nbPolar]
    %   cm     : Array with moment coefficients [aoas x nbPolar]
    %
    % Output:
    %   Various plots
    %
    % Example:
    %   PLOTPOLARS
    %   PLOTPOLARS(Polar)
    %   PLOTPOLARS(alpha, cl)
    %   PLOTPOLARS(alpha, cl, cd)
    %   PLOTPOLARS(alpha, cl, cd, cm)
    %
    % See also: XF2MAT, EXTENDPOLAR.
    %
    % <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (online)</a>

    % -------------------------------------------
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
    % Issues: https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues
    % ----------------------------------------------------------------------------------------------

    narginchk(0, 4);

    % Import other functions from this package
    import af_tools.utils.*

    % Defaults and constants
    OPTION_LIST = {};   % No options for this function

    % Extract and validate the inputs
    [alpha, c_l, c_d, c_m] = parsepolarinputs(OPTION_LIST, varargin{:});

    % -------------------------------------------
    % Plots

    set(0, 'defaultTextInterpreter', 'latex'); % Default interpreter for all plots

    plotsinglepolar(rad2deg(alpha), c_l, 'C_l - \alpha', '$\alpha$ [deg]', '$C_l$ [-]');
    if nargin > 2 || nargin <= 1
        plotsinglepolar(rad2deg(alpha), c_d, 'C_d - \alpha', '$\alpha$ [deg]', '$C_d$ [-]');
        plotsinglepolar(rad2deg(alpha), c_l ./ c_d, 'C_l/C_d - \alpha', '$\alpha$ [deg]', ...
                        '$C_l/C_d$ [-]');
        plotsinglepolar(c_l, c_d, 'C_d - C_l', '$C_l$ [-]', '$C_d$ [-]');
    end
    if nargin > 3 || nargin <= 1
        plotsinglepolar(rad2deg(alpha), c_m, 'C_m - \alpha', '$\alpha$ [deg]', '$C_m$ [-]');
    end

end

% --------------------------------------------------------------------------------------------------
function plotsinglepolar(dataX, dataY, fTitle, fXlabel, fYlabel)
    % PLOTSINGLE Standardize polar plots

    if ~isempty(dataX) && ~isempty(dataY)

        if nargin < 3
            fTitle = '';
        end

        figure('PaperUnits', 'inches', 'PaperPosition', [0 0 1280 1024] / 250, 'Name', fTitle);
        plot(dataX, dataY);

        if nargin >= 4
            xlabel(fXlabel);
        end
        if nargin >= 5
            ylabel(fYlabel);
        end

        setgca();
    end

end

% --------------------------------------------------------------------------------------------------
function setgca()
    % SETGCA Sets axes options for all plots

    grid on;
    set(gca, 'FontName', 'Helvetica');
    set(gca, 'Fontsize', 14);
    set(gca, ...
        'Box', 'off', ...
        'TickDir', 'out', ...
        'TickLength', [.02 .02], ...
        'XMinorTick', 'off', ...
        'YMinorTick', 'off', ...
        'YGrid', 'on', ...
        'XGrid', 'on', ...
        'XColor', 'k', ...
        'YColor', 'k', ...
        'GridLineStyle', ':', ...
        'GridColor', 'k', ...
        'GridAlpha', 0.25, ...
        'LineWidth', 1);
    set(gcf, 'PaperPositionMode', 'auto');

end
