# Contributing

When contributing to this repository, please first discuss the change you wish
to make via issue, email, or any other before making a change.

- [Code standards](#code-standards)
  - [Documentation](#documentation)
  - [Inputs and errors](#inputs-and-errors)
  - [Tests](#tests)
  - [Style](#style)
    - [MISS_HIT](#miss_hit)
    - [Other](#other)
    - [Naming convention](#naming-convention)
- [Merge request process](#merge-request-process)

## Code standards

All changes submitted must conform to the following code standards.

### Function design

The functions of this packages are divided in two parts: the _main functions_
and the _utilities_. The _main functions_ are the ones present at the root of
`+af_tools/`. These are the core functions of this package, meant to be called
directly by the users. The _utilities_ are located in `+af_tools/+utils/`. These
are small sub-routines, meant to perform basic tasks and shared among the
different _main functions_. These sub-routines are not intended to be called by
the end user (although they could if they want). Therefore they do not require
the same level of documentation, testing, etc.

- Each main function should be standalone tool that makes a complete operation.
- If possible, the main functions should be coded so they can be run without any
  input argument. If some argument is absolutely mandatory, the users should be
  prompted through the command window or a dialog box. All other arguments
  should be set at proper defaults values (documented in the help block).
- The functions must be inter-operable with little effort from the user
  (function B reuses the output of function A without much modification). This
  means that the units of the input and output should remain the same, etc.
- The functions can be called directly from one another. In that situation, the
  import statements must be the first lines of code in the function.

### Documentation

- Every _main function_ should start with a lengthy comment that details its
  use. The first line of the comment must be a one-line description of the
  function. The first block of continuous comment is printed when the user types
  `help <function>`. This is the main documentation for the different functions,
  therefore it should be as complete as possible. This comment must document the
  function's aim, its inputs and outputs. Various small examples should be also
  provided.
  If some special design choices were made for the implementation, they should
  be documented in detail well.
- If the function implements a specific methodology, please add the source as
  well in the documentation.
- The first comment block should always end with a copyright block detailing the
  author and the license (Apache 2.0).
- The _utilities_ functions can just have a one-line description and the
  copyright block. They should normally be simple enough to be understood
  without much work.
- All _main functions_ should be briefly described in the `README.md` of this
  repository. This small description should detail its aim, inputs and a few
  examples.

### Inputs and errors

- Every _main function_ should validate completely the user inputs.
- This validation should be done using Matlab's built-in tools as much as
  possible. Such tools are: `validateattributes`, `validatestring`, `try/catch`
  blocks, etc. Some utilities functions already exist to take care of input
  parsing. Ideally, your function should try to reuse them as is to standardize
  things as much as possible.
- The sub-routines can skip these user input checks, as they are meant to be
  used internally.

- All functions should return a comprehensive error message in case of failure.
  This message is meant to help the user debug their issue on themselves.
- If applicable, warnings should be returned if the inputs/outputs are
  expected to be of poor quality due to poor, but valid, input.
- In order to facilitate testing, the error and warnings must come with a custom
  identifier and not just a description.

### Tests

- Each function (whereas _main function_ or _utility_) **must** come with a set
  of tests, placed in the `tests/` directory.
- These tests typically check:
  1. if errors and warning are returned as expected;
  2. a few basic use cases.
- For _utilities_, the errors and warning tests can be skipped (typically the
  error should be raised beforehand in the main function that calls the utility).

### Style

#### Miss_hit

The code style is enforced using [MISS_HIT][miss_hit], a code formatter and
analyser written in Python. **All pushes to the master branch and all merge
requests are first validated using MISS_HIT**.

Besides typical style verification (spaces at the right spot, line width, etc),
MISS_HIT is also used to analyze the complexity of the code. To simplify
maintenance and interoperability, each individual function should be kept as
small and simple as possible. Do not hesitate to use small local functions to do
basic operations within a larger function.

To install MISS_HIT, just use the following:

```bash
pip3 install --user miss_hit
```

The rules enforced by MISS_HIT are specified in the `miss_hit.cfg` [file at the
root of this repo][miss_hit_cfg].

To check the code style of the entire repository, just execute the following
line at the root of the repo.

```bash
mh_style .
```

You can also ask MISS_HIT to fix everything automatically by simply adding the
flag `--fix` to the bash command.

The code quality metrics can be checked with

```bash
mh_metrics .
```

The output is quite verbose as all metrics for all files is returned. Adding
the flag `--ci` will only list the metrics that conflict with the limits imposed
in the configuration file.

MISS_HIT also comes with a linting tool that will typically output warnings, etc
when some variables are poorly defined, or code may lead to future breaking
errors.

```bash
mh_lint .
```

A more convenient way to use MISS_HIT is to include it in a pre-commit hook.
This can be done with [pre-commit][pre-commit-site]. To install both
dependencies just run:

```bash
pip3 install -r requirements.txt
```

The install the pre-commit hooks using:

```bash
pre-commit install
```

Now the three MISS_HIT checks will be run every time you make a commit.

#### Other

Some other conventions can not be currently enforced by MISS_HIT. They should
still apply:

- The functions should not contain commented code.
- The functions should not be separated in code blocks using `%%`. Prefer a
  separator line such as `% --------------------------`

#### Naming convention

The following naming convention is adopted:

- File names must be the same as the main function name.
- Functions must be _alllowercase_: `calculatestuff()`, `getallvariables()`
- Constants should be all caps, with underscore between words: `MAX_ITERATIONS`
- In general, variables should use _camelCase_: `myVariable`
  - Matlab structures should be _PascalCase_: `MyStructure`
  - Indices are allowed if they make sense: `alpha_0`, `cDrag_0`, `alpha_s`
  - Aerodynamic coefficients must have an underscore: `c_d` (prevents conflict
    with `cd` command)

## Merge Request Process

1. Ensure the **code standards** are properly respected (function description,
   help comment, unit tests, inter-operability, etc).
2. Ensure all tests pass in Matlab (`runtests('tests')`).
3. Update the CHANGELOG.md and CODEOWNERS to document the changes.
4. Update the README.md to reflect the changes if applicable.
5. Ensure MISS_HIT is happy
6. Submit your Merge Request.

[pre-commit-site]: https://pre-commit.com/
[miss_hit]: https://github.com/florianschanda/miss_hit
[miss_hit_cfg]: https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/blob/master/miss_hit.cfgj:w
