% TEST_EXTENDPOLAR Unitary tests for the extendpolar function
% Note:
%   Matlab package functionality is not very test-framework friendly.
%   Either the whole package has to be imported in EVERY SINGLE TEST or the
%   functions must be called as <package>.function() everytime.
%   The second option is preferred for the tests as it has the smaller scope.
%
% -----
% (c) Copyright 2022 University of Liege
% Author: Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox

% --------------------------------------------------------------------------------------------------

%% Main test function

function tests = test_extendpolar

    tests = functiontests(localfunctions);

end

%% Setup and teardown

function setupOnce(testCase)

    addpath('../.'); % Add repository to Matlab Path
    addpath('./test_utils'); % Add utils to Matlab Path

    % Set random number generator settings.
    testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)

    rmpath('../.'); % Remove repository from Matlab Path
    rmpath('./test_utils'); % Remove utils from Matlab Path

    % Restore the random number generator settings.
    s = testCase.TestData.currentRNG;
    rng(s);

end

function teardown(~)

    close all; % Close all figures that would have been openend

end

%% Arguments and errors tests

function test_nargin(testCase)
    % Error if too many arguments

    dummy = (1:10)';

    verifyError(testCase,  ...
                @() af_tools.extendpolar(dummy, dummy, dummy, dummy, dummy, 'method', 'viterna', 'limit', 'stall'),  ...
                'MATLAB:narginchk:tooManyInputs');

end

function test_invalidOptions(testCase)
    % Error if invalid option name or missing parameter value

    Dummy = dummysinglepolar();
    wrongName = 'wrongoption';

    verifyError(testCase, @() af_tools.extendpolar(Dummy, wrongName), ...
                'MATLAB:InputParser:ParamMissingValue');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, wrongName, true), ...
                'MATLAB:InputParser:UnmatchedParameter');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'method', 'Viterna', wrongName), ...
                'MATLAB:InputParser:ParamMissingValue');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'method', 'Viterna', wrongName, true), ...
                'MATLAB:InputParser:UnmatchedParameter');

end

function test_invalidOptionsVal(testCase)
    % Error if valid option name but invalid value

    Dummy = dummysinglepolar();
    empty = [];
    scal = 0;
    char = 'test';
    Struct = struct;
    vect = 1:10;

    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'method', empty), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'method', scal), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'method', char), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'method', Struct), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'method', vect), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');

    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'limit', empty), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'limit', scal), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'limit', char), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'limit', Struct), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');
    verifyError(testCase, @() af_tools.extendpolar(Dummy, 'limit', vect), ...
                'MATLAB:extendpolar:unrecognizedStringChoice');

end

% ------------------------------------------------------------------------------
function test_noStall(testCase)
    % Error if impossible to find the stall

    Polar = dummysinglepolar();
    Polar.cl = ones(size(Polar.cl));

    verifyError(testCase, @() af_tools.extendpolar(Polar), 'MATLAB:extendpolar:noStall');

end

function test_noZero(testCase)
    % Error if impossible to find the stall

    Polar = dummysinglepolar();
    % Shift alpha so it is all positive
    Polar.aoa = Polar.aoa(:) + abs(Polar.aoa(1)) + 0.1;

    verifyError(testCase, @() af_tools.extendpolar(Polar), 'MATLAB:extendpolar:noZero');

end

%% Test if function returns expected outputs

function test_correctAoaSizeSinglePolar(testCase)
    % Verify if the output aoa has the correct size
    % Should not re-expand polar that already covers the full range

    Polar = dummysinglepolar();
    stepAoa = min(diff(Polar.aoa));
    fullRange = (-pi:stepAoa:pi)';

    [aoaExt, clExt, cdExt] = af_tools.extendpolar(Polar);
    [aoaExt2, clExt2, cdExt2] = af_tools.extendpolar(aoaExt, clExt, cdExt);

    verifyEqual(testCase, aoaExt, fullRange, 'AbsTol', 1e-9);

    verifyEqual(testCase, aoaExt2, aoaExt);
    verifyEqual(testCase, clExt2, clExt);
    verifyEqual(testCase, cdExt2, cdExt);

    verifyEqual(testCase, size(clExt2), size(aoaExt));
    verifyEqual(testCase, size(cdExt2), size(aoaExt));

end

function test_correctAlphaSizeDoublePolar(testCase)
    % Verify if the output alpha has the correct size
    % Should not re-expand polar that already covers the full range

    Polar = dummydoublepolar();
    stepAlpha = min(diff(Polar.aoa));
    fullRange = (-pi:stepAlpha:pi)';

    [alphaExt, clExt, cdExt] = af_tools.extendpolar(Polar);
    [alphaExt2, clExt2, cdExt2] = af_tools.extendpolar(alphaExt, clExt, cdExt);

    verifyEqual(testCase, alphaExt(:, 1), fullRange, 'AbsTol', 1e-9);

    verifyEqual(testCase, alphaExt2, alphaExt);
    verifyEqual(testCase, clExt2, clExt);
    verifyEqual(testCase, cdExt2, cdExt);

    verifyEqual(testCase, size(clExt2), size(alphaExt));
    verifyEqual(testCase, size(cdExt2), size(alphaExt));

end
