% TEST_findzerolift Unitary tests for the findzerolift function
% Note:
%   Matlab package functionality is not very test-framework friendly.
%   Either the whole package has to be imported in EVERY SINGLE TEST or the
%   functions must be called as <package>.function() everytime.
%   The second option is preferred for the tests as it has the smaller scope.
%
% -----
% (c) Copyright 2022 University of Liege
% Author: Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox

% --------------------------------------------------------------------------------------------------

%% Main test function

function tests = test_findzerolift

    tests = functiontests(localfunctions);

end

%% Setup and teardown

function setupOnce(testCase)

    addpath('../.'); % Add repository to Matlab Path
    addpath('./test_utils'); % Add utils to Matlab Path

    % Set random number generator settings.
    testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)

    rmpath('../.'); % Remove repository from Matlab Path
    rmpath('./test_utils'); % Remove utils from Matlab Path

    % Restore the random number generator settings.
    s = testCase.TestData.currentRNG;
    rng(s);

end

function teardown(~)

    close all; % Close all figures that would have been openend

end

%% Arguments and errors tests

function test_nargin(testCase)
    % Error if too many arguments

    dummy = (1:10)';

    verifyError(testCase, @() af_tools.findzerolift(dummy, dummy, dummy, dummy), 'MATLAB:narginchk:tooManyInputs');

end

function test_noPositiveCl(testCase)
    % Error if Cl is never positive (impossible to find zero)

    Polar = dummysinglepolar();
    Polar.cl = -ones(size(Polar.cl));
    verifyError(testCase, @() af_tools.findzerolift(Polar), 'MATLAB:findzerolift:noPositiveCl');

end

function test_noNegativeCl(testCase)
    % Error if Cl is never negative (impossible to find zero)

    Polar = dummysinglepolar();
    Polar.cl = ones(size(Polar.cl));
    verifyError(testCase, @() af_tools.findzerolift(Polar), 'MATLAB:findzerolift:noNegativeCl');

end

%% Test if function returns expected outputs

function test_correctResults(testCase)
    % Check if function works as intended on basic cases

    alphad = (-10:10)';
    alpha = deg2rad(alphad);
    c_l = alphad - 5;
    c_d = alphad + 5;

    [alpha_0, cd_0] = af_tools.findzerolift(alpha, c_l, c_d);

    verifyEqual(testCase, alpha_0, deg2rad(5));
    verifyEqual(testCase, cd_0, 10);
end
