% TEST_PARSENACAINPUTS Unitary tests for the parsenacainputs sub-function
%   Due to the importance of this small utility function, many tests were
%   written. This should ensure maximum complicance in all possible context.
% Note:
%   Matlab package functionality is not very test-framework friendly.
%   Either the whole package has to be imported in EVERY SINGLE TEST or the
%   functions must be called as <package>.function() everytime.
%   The second option is preferred for the tests as it has the smaller scope.
%
% -----
% (c) Copyright 2022 University of Liege
% Author: Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox

% --------------------------------------------------------------------------------------------------

%% Main test function

function tests = test_parsenacainputs

    tests = functiontests(localfunctions);

end

%% Setup and teardown

function setupOnce(testCase)

    addpath('../.'); % Add repository to Matlab Path
    addpath('./test_utils'); % Add utils to Matlab Path

    % Set random number generator settings
    testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)

    rmpath('../.'); % Remove repository from Matlab Path
    rmpath('./test_utils'); % Remove utils from Matlab Path

    % Restore the random number generator settings
    s = testCase.TestData.currentRNG;
    rng(s);

end

function teardown(~)

    close all; % Close all figures that would have been openend

end

%% Check if errors and warnings are raised

function test_invalidOptsList(testCase)
    % Error if optList is not a cell array or a string array

    empty = [];
    scal = 0;
    logi = true;
    Struct = struct;

    % OptList must be a char, string or cell array
    verifyError(testCase, @() af_tools.utils.parsenacainputs(empty), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(scal), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(logi), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(Struct), 'MATLAB:parsenacainputs:invalidType');

end

function test_invalidInputTypeWhenNoOpts(testCase)
    % Error if any input is of the wrong type

    EMPTY_OPTS = {};
    empty = [];
    scal = 0;
    logi = true;
    char = 'test';
    Struct = struct;
    vect = 1:10;

    % 1. Digits (char or str)
    verifyError(testCase, @() af_tools.utils.parsenacainputs(EMPTY_OPTS, empty), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(EMPTY_OPTS, scal), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(EMPTY_OPTS, logi), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(EMPTY_OPTS, Struct), 'MATLAB:parsenacainputs:invalidType');

    % 2. nPoints (numeric, above 2)
    verifyError(testCase, @() af_tools.utils.parsenacainputs(EMPTY_OPTS, '0012', empty), ...
                'MATLAB:parsenacainputs:expectedScalar');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(EMPTY_OPTS, '0012', logi), ...
                'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(EMPTY_OPTS, '0012', char), ...
                'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(EMPTY_OPTS, '0012', Struct), ...
                'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(EMPTY_OPTS, '0012', vect), ...
                'MATLAB:parsenacainputs:expectedScalar');

end

function test_invalidDigitInputWhenNoOpts(testCase)
    % Error if digits is not an actual NACA 4 or 5 digit

    EMPTY_OPTS = {};

    verifyError(testCase, @() af_tools.utils.parsenacainputs(EMPTY_OPTS, '012'), ...
                'MATLAB:parsenacainputs:wrongNumberOfDigits');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(EMPTY_OPTS, '012345'), ...
                'MATLAB:parsenacainputs:wrongNumberOfDigits');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(EMPTY_OPTS, 'oo12'), ...
                'MATLAB:parsenacainputs:invalidDigits');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(EMPTY_OPTS, '24312'), ...
                'MATLAB:parsenacainputs:invalid5Digits');

end

function test_invalidInputTypeWithOpts(testCase)
    % Error if any input is of the wrong type

    OPTS = {'option1', 'option2'};
    empty = [];
    scal = 0;
    logi = true;
    char = 'test';
    Struct = struct;
    vect = 1:10;

    % 1. Digits (char or str)
    verifyError(testCase, @() af_tools.utils.parsenacainputs(OPTS, empty), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(OPTS, scal), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(OPTS, logi), 'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(OPTS, Struct), 'MATLAB:parsenacainputs:invalidType');

    % 2. nPoints (numeric, above 2)
    verifyError(testCase, @() af_tools.utils.parsenacainputs(OPTS, '0012', empty), ...
                'MATLAB:parsenacainputs:expectedScalar');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(OPTS, '0012', logi), ...
                'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(OPTS, '0012', char), ...
                'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(OPTS, '0012', Struct), ...
                'MATLAB:parsenacainputs:invalidType');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(OPTS, '0012', vect), ...
                'MATLAB:parsenacainputs:expectedScalar');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(OPTS, '0012', 1), ...
                'MATLAB:parsenacainputs:notGreaterEqual');

end

function test_invalidDigitInputWithOpts(testCase)
    % Error if digits is not an actual NACA 4 or 5 digit

    OPTS = {'option1', 'option2'};

    verifyError(testCase, @() af_tools.utils.parsenacainputs(OPTS, '012'), ...
                'MATLAB:parsenacainputs:wrongNumberOfDigits');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(OPTS, '012345'), ...
                'MATLAB:parsenacainputs:wrongNumberOfDigits');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(OPTS, 'oo12'), ...
                'MATLAB:parsenacainputs:invalidDigits');
    verifyError(testCase, @() af_tools.utils.parsenacainputs(OPTS, '24312'), ...
                'MATLAB:parsenacainputs:invalid5Digits');

end

%% Test if function returns expected outputs

function test_noNPoints(testCase)
    % Check if NPOINTS are assigned to 100 when not specified

    EMPTY_OPTS = {};

    [digits, nPoints, idxOpts] = af_tools.utils.parsenacainputs(EMPTY_OPTS, '0012');

    verifyEqual(testCase, digits, '0012');
    verifyEqual(testCase, nPoints, 100);
    verifyEqual(testCase, idxOpts, 2);

end

function test_normalInputsWithoutOpts(testCase)
    % Check if inputs are properly returned

    EMPTY_OPTS = {};

    [digits, nPoints, idxOpts] = af_tools.utils.parsenacainputs(EMPTY_OPTS, '2412', 12);

    verifyEqual(testCase, digits, '2412');
    verifyEqual(testCase, nPoints, 12);
    verifyEqual(testCase, idxOpts, 3);

end

function test_normalInputsWithOptsNotInArgs(testCase)
    % Check if inputs are properly returned

    EMPTY_OPTS = {'option'};

    [digits, nPoints, idxOpts] = af_tools.utils.parsenacainputs(EMPTY_OPTS, '2412', 12);

    verifyEqual(testCase, digits, '2412');
    verifyEqual(testCase, nPoints, 12);
    verifyEqual(testCase, idxOpts, 3);

end

function test_normalInputsWithOptsInArgs(testCase)
    % Check if inputs are properly returned

    EMPTY_OPTS = {'option'};

    [digits, nPoints, idxOpts] = ...
      af_tools.utils.parsenacainputs(EMPTY_OPTS, '2412', 133, 'option', true);

    verifyEqual(testCase, digits, '2412');
    verifyEqual(testCase, nPoints, 133);
    verifyEqual(testCase, idxOpts, 3);

end
